<?php
/**
 * Criptografia. Classe com diversos mtodos para criptografar textos e arquivos
 *
 */
class Criptografia
{
	private $_algoritmo;
	private $_modoBinario;
	private $_nums = array('0', '1', '2', '3', '4', '5');
	private $_letr = array('a', 'b', 'c', 'd', 'e', 'f');

	//--------------------------------------------------------------------------
	/**
	 * Construtor. Pode-se definir o algoritmo de hashing usado para a
	 * a criptografia dos dados. O mais seguro que existe hoje  o sha512, que 
	 * o padro. Para mudar o algoritmo para o md5 p.ex.  s especific-lo no
	 * primeiro parmetro do construtor. O modo binrio  um meio de transformar
	 * a string em caracteres binrios no legveis. Pode ser til para cifrar o
	 * contedo de um arquivo em um outro arquivo.
	 *
	 * @param String $algoritmo Algoritmo de hashing preferencial
	 * @param boolean $modoBinario Modo binrio no legvel
	 */
	public function __construct($algoritmo = 'sha512', $modoBinario = false)
	{
		if( !in_array( $algoritmo, hash_algos() ) ) {

			die("Algoritmo '$algoritmo' no disponvel neste sistema.");			
		}
		
		if( $modoBinario == true ) {
			
			if(!function_exists('gzdeflate') || !function_exists('gzinflate')) {
				
				die('Modo binrio no disponvel neste sistema');
			}
		}
		
		$this->_algoritmo = $algoritmo;
		$this->_modoBinario = $modoBinario;
		
	}
	//--------------------------------------------------------------------------
	/**
	 * Cifrar. Mtodo que gera um texto cifrado de linha nica, representado em
	 * caracteres "hexadecimais" disfarados. Se o modo binrio estiver ativo,
	 * o texto no ser mais disposto em caracteres hexadecimais legveis. A
	 * chave secreta  opcional, mas  mais seguro us-la, pois s poder
	 * decifrar o contedo do texto cifrado quem conhecer e informar a chave
	 * para o mtodo que decifra.
	 *
	 * @param String $texto Texto limpo (no cifrado)
	 * @param String $chave Chave para cifrar o texto (opcional)
	 * @return String Texto cifrado
	 */
	public function Cifrar($texto, $chave = 'chave')
	{
		if( $this->_modoBinario ) {
            $ch = strrev( hash($this->_algoritmo, $chave) );
        }
		else {
            $ch = strrev( hash($this->_algoritmo, $chave . strlen($texto)) );
        }
		
		$tamTexto = strlen($texto);
		$tamChave = strlen($ch);
		
		$textoCifrado = '';
		
		for($i=0, $j=0; $i<$tamTexto; $i++) {
			
			if( $j < ($tamChave - 1)) $j++;
			else $j = 0;
			
			$textoCifrado .= sprintf('%03d', (ord($texto[$i]) + ord($ch[$j])));
			
		}

		if($this->_modoBinario) {
			
			return gzdeflate(str_replace($this->_nums, $this->_letr,
				$textoCifrado) );
		}
		
		return str_replace($this->_nums, $this->_letr, $textoCifrado);
	}
	//--------------------------------------------------------------------------
	/**
	 * Decifrar. Mtodo responsvel por decifrar o contedo do texto cifrado
	 * pelo mtodo Cifrar() desta classe. O texto s poder ser decifrado com o
	 * conhecimento da chave, ou se a chave no foi usada na operao de cifra.
	 *
	 * @param String $textoCifrado Texto cifrado
	 * @param String $chave Chave para decifrar o texto (opcional)
	 * @return String Texto decifrado
	 */
	public function Decifrar($textoCifrado, $chave = 'chave')
	{
		if($this->_modoBinario) {
            
            $ch = strrev( hash($this->_algoritmo, $chave) );
			$textoCifrado = gzinflate($textoCifrado);
		}
        else {
        
            $ch = strrev( hash($this->_algoritmo, 
                          $chave . ( strlen($textoCifrado)/3 ) ) );
        }

		
		$texto = str_replace($this->_letr, $this->_nums, $textoCifrado);
		
		
		$tamTexto = strlen($texto);
		$tamChave = strlen($ch);
		
		$textoDecifrado = '';
		
		$arrayDeCaracteres = str_split($texto, 3);
		
		for($i=0, $j=0; $i < (($tamTexto / 3)); $i++) {
			
			if( $j < ($tamChave - 1) ) $j++;
			else $j = 0;
			
			$textoDecifrado .= chr($arrayDeCaracteres[$i] - ord($ch[$j]) );
		}
		
		return $textoDecifrado;	
	}
	//--------------------------------------------------------------------------
	/**
	 * Senha. Mtodo usado para criptografar uma senha que contenha no mnimo um
	 * caracter e no mximo dez caracteres. A senha criptografada, seja qual for
	 * o seu tamanho (compreendido entre um e dez caracteres) ir ser gerada com
	 * trinta e dois caracteres em formato "hexadecimal" disfarado. O mesmo
	 * mtodo servir para criptografar ou descriptografar a senha. Uma senha
	 * criptografada sempre conter trinta e dois caracteres, nos quais os dois
	 * ltimos representam o nmero de caracteres reais da senha informada. Para
	 * decriptar a senha e exib-la novamente conforme foi digitada, basta
	 * aplicar este mesmo mtodo na senha criptografada.
	 *
	 * @param String $senha Senha limpa ou criptografada
	 * @return String Senha limpa ou criptografada
	 */
	public function Senha($senha)
	{
		$tamanhoDaSenha = sprintf('%02d', strlen($senha) );
		
		// A senha no pode ter mais de 10 caracteres, infelizmente
		if(((int)$tamanhoDaSenha > 10 && (int)$tamanhoDaSenha != 32) ||
			(int)$tamanhoDaSenha == 0) {
			
			return false;
		}
		
		// Se a senha  a senha crua, cifrar a mesma:
		if( (int)$tamanhoDaSenha <= 10 ) {
			
			$ch = strrev( hash($this->_algoritmo, $tamanhoDaSenha) );
			
			$senhaCifrada = '';
			
			for($i=0; $i<(int)$tamanhoDaSenha; $i++) {
				
				$senhaCifrada .= sprintf('%03d', (ord($senha[$i]) + ord($ch[$i])));
				
			}
	
			$senhaCifrada = str_replace($this->_nums, $this->_letr, $senhaCifrada);
			
			// Completando com valores de onde termina a senha at 30 caracteres

			$completude = hash($this->_algoritmo, $senha);
			
			for($i=(strlen($senhaCifrada) - 1); $i<29; $i++) {
				
				$senhaCifrada .= $completude[$i];
				
			}
			
			return $senhaCifrada . $tamanhoDaSenha;
		}
		
		// Se a senha  a senha cifrada, decifrar a mesma:
		else {
			
			$tamanhoDaSenha = substr($senha, 30);
			
			$ch = strrev( hash($this->_algoritmo, $tamanhoDaSenha) );
			
			$senhaSemLixo = substr($senha, 0, ( (int)$tamanhoDaSenha * 3) );
			
			$senhaNums = str_replace($this->_letr, $this->_nums, $senhaSemLixo);
			
			$senhaDecifrada = '';
			
			$arrayDeCaracteres = str_split($senhaNums, 3);
			
			for($i=0; $i<(int)$tamanhoDaSenha; $i++) {
				
				$senhaDecifrada .= chr($arrayDeCaracteres[$i] - ord($ch[$i]) );
			}
			
			return $senhaDecifrada;
		}
	}
	//--------------------------------------------------------------------------
	/**
	 * GeraSenha. Mtodo que gera uma senha aleatria com no mximo dez
	 * caracteres.
	 *
	 * @param int $tamanhoMinimo Tamanho mnimo de caracteres para a senha
	 * @return String Senha gerada
	 */
	public function GerarSenha($tamanhoMinimo = 3)
	{
		return substr( str_shuffle( uniqid() ), 0, mt_rand($tamanhoMinimo, 10));
	}
	//--------------------------------------------------------------------------
	/**
	 * AlgoritmosDisponiveis. Exibe uma lista completa de algoritmos disponveis
	 * no sistema para o uso nesta classe. At o presente momento  recomendado
	 * o uso do sha512, que  o mais seguro.
	 */
	public function AlgoritmosDisponiveis()
	{
		$algoritmos = hash_algos();
		
		if( $total = count($algoritmos) ) {
			
			echo "<fieldset><legend>Algoritmos - total: $total</legend>";
			foreach ($algoritmos as $algoritmo) {
				
				$tam = strlen( hash($algoritmo, 'a') );
				echo "<li>$algoritmo - Tamanho da string: $tam</li>";
			}
			echo '</fieldset>';
		}
	}
	//--------------------------------------------------------------------------
}