<?php


/*
 *  Sivac - Sistema Online de Vacinao     
 *  Copyright (C) 2012  IPPES  - Institituto de Pesquisa, Planejamento e Promoo da Educao e Sade   
 *  www.sivac.com.br                     
 *  ippesaude@uol.com.br                   
 *                                                                    
 *  Este programa e software livre; voc pode redistribui-lo e/ou     
 *  modific-lo sob os termos da Licena Publica Geral GNU, conforme  
 *  publicada pela Free Software Foundation; tanto a verso 2 da      
 *  Licena como (a seu critrio) qualquer verso mais nova.          
 *                                                                    
 *  Este programa  distribudo na expectativa de ser til, mas SEM   
 *  QUALQUER GARANTIA; sem mesmo a garantia implcita de              
 *  COMERCIALIZACAO ou de ADEQUACAO A QUALQUER PROPOSITO EM           
 *  PARTICULAR. Consulte a Licena Publica Geral GNU para obter mais  
 *  detalhes.                                                         
 *                                                                    
 *  Voc deve ter recebido uma copia da Licena Publica Geral GNU     
 *  junto com este programa; se no, escreva para a Free Software     
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA          
 *  02111-1307, USA.                                                  
 *  
 *  Copia da licena no diretrio Sistema/licenca_en.txt 
 *                                Sistema/licenca_pt.txt 
 */


/**
 * Simac_Bd_Preparacao: Prepara os dados para serem inseridos sem smbolos no BD
 *
 * Esta classe prepara os dados, tanto para serem inseridos no banco de dados,
 * removendo smbolos como pontos, traos, barras, etc. mantendo somente os
 * dados relevantes, como prepara os dados recuperados do banco de dados para
 * serem exibidos para o usurio em um formato "legvel" - com pontos e traos.
 *
 * @package Sivac/Class
 *
 * @author Douglas, v 1.0, 2008-07-23 10:36
 *
 * @copyright 2008 
 */
//-----------------------------------------------------------------------------
class Preparacao
{
	/**
	 * Smbolos que sero removidos quando um CPF, CNPJ, Tel, etc. forem
	 * inseridos no banco de dados. Os traos, pontos etc. so exibidos na tela
	 * com auxlio do javascript, porm no banco de dados s so armazenadas as
	 * informaes essenciais. No caso de um nmero de IP, os pontos sero
	 * armazenados, pois no teramos como diferenciar p.ex. 200.122.23.122 de
	 * 200.12.223.122 ou outras variaes.
	 *
	 * @staticvar array $simbolos alguns caracteres que sero removidos antes da
	 * insero no banco de dados.
	 */
	private static $simbolos = array('-', '.', '/', '(', ')', ',', ':');
	//--------------------------------------------------------------------------
	/**
	 * Gera um array limpo, baseado no input do usurio - que vem do array
	 * $_POST, que captura as variveis do formulrio. Este mtodo  necessrio
	 * para certificar que no h seqncias maliciosas de cdigo fornecidas
	 * pelo usurio. Caso tenha, o mtodo devolver um array "limpo" (sem essas
	 * seqncias ou com essas seqncias "escapadas" para no produzirem efeito
	 * malicioso ao serem interpretadas).
	 *
	 * @param array $post Array ainda no tratado nem filtrado
	 * @return array|null Um array "limpo", escapado, sem espaos em branco, sem os
	 * smbolos usados na formatao e "seguro" de acordo com o
	 * mysqli->real_scape_string()
	 */
	public static function GerarArrayLimpo($post, $conexao = false)
	{

        // o "new mysqli" foi adicionado aqui prq ele no estava conseguindo
        // se conectar e tava dando erro. Inicialmente isso funcionava sem o "new mysqli",
        // mas foi feita alguma modificao desconhecia que acabou afetando isso.
        $conexao = new mysqli($_SERVER['HOST_SIVAC'],
                      $_SERVER['USER_SIVAC'],
                      $_SERVER['PASS_SIVAC'],
                      $_SERVER['BD_SIVAC'],
                      $_SERVER['PORT_SIVAC']);
        
		if( !$conexao ) $conexao = mysqli_connect();

        if(  !count($post)  ) return null; // se o post estiver vazio, retorna

		$clean = array();

		foreach ($post as $chave => $valor)
		{
			$clean[$chave] = $conexao->real_escape_string(trim($valor));
		}
		return $clean;
	}
	//--------------------------------------------------------------------------
	/**
	 * Mtodo que insere os smbolos para a melhor visualizao do usurio nos
	 * campos como telefone, cpf, cnpj, etc. Os smbolos (pontos, traos, barras,
	 * etc.) so reinseridos, pois no banco de dados s so inseridas as
	 * informaes estitamente necessrias. Os smbolos, alm de ocupar
	 * espao desnecessrio poder atrapalhar em alguns casos. Logo, um telefone
	 *  armazenado como "45895555" ao invs de "4589-5555" e um CPF como
	 * "65445428737" ao invs de "654.454.287-37".
	 *
	 * @param string $campo O campo "cr" - sem smbololos, recuperado do banco
	 * @param string $tipo Telefone, CPF, CNPJ, etc.
	 * @return string $campoFormatado Campo formatado com smbolos
	 */
	public static function InserirSimbolos($campo, $tipoDeCampo)
	{
		$campoFormatado = '';

		$tipo = strtoupper($tipoDeCampo);

		switch ($tipo) {
			case 'CPF':
				$campoFormatado = substr($campo, 0, 3)
								. '.' . substr($campo, 3, 3)
								. '.' . substr($campo, 6, 3)
								. '-' . substr($campo, 9);
				break;

			case 'CNPJ':
				$campoFormatado = substr($campo, 0, 2)
								. '.' . substr($campo, 2, 3)
								. '.' . substr($campo, 5, 3)
								. '/' . substr($campo, 8, 4)
								. '-' . substr($campo, 12);
				break;

			case 'TEL':
				$campoFormatado = substr($campo, 0, 4)
								. '-' . substr($campo, 4);
				break;
			case 'INSC':
				$campoFormatado = substr($campo, 0, 2)
								. '.' . substr($campo, 2, 3)
								. '.' . substr($campo, 5);
				break;
			case 'CEP':
				if( strlen($campo) >= 8 )
				$campoFormatado = substr($campo, 0, 5)
								. '-' . substr($campo, 5);
				else $campoFormatado = '';
				break;

			default:
				$campoFormatado = $campo;
		}

		return $campoFormatado;
	}

	//--------------------------------------------------------------------------
	/**
	 * Mtodo que remove os smbolos (pontos, traos, barras, etc.)para a
	 * insero no banco de dados.
	 *
	 * @param String $campo
	 * @return String $campo
	 */
	public static function RemoverSimbolos($campo) {

		return str_replace(self::$simbolos, '', $campo);
	}
}
/*
Para teste:

$cpf = '07582493737';
$cnpj = '60659463000191';
$tel = '25662489';
$insc = '52658965';
$cep = '28660000';
$outro = '123567890';

echo '<pre>',
	Simac_Bd_Preparacao::InserirSimblos($cpf, 'cpf'), PHP_EOL,
	Simac_Bd_Preparacao::InserirSimblos($cnpj, 'CNpj'), PHP_EOL,
	Simac_Bd_Preparacao::InserirSimblos($tel, 'tEL'), PHP_EOL,
	Simac_Bd_Preparacao::InserirSimblos($insc, 'insc'), PHP_EOL,
	Simac_Bd_Preparacao::InserirSimblos($cep, 'cep'), PHP_EOL,
	Simac_Bd_Preparacao::InserirSimblos($outro, 'algo'), PHP_EOL;
*/